//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.lib.locale;

import java.util.EnumMap;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import net.ajaest.lib.locale.SwingMessagesEnums.ContextMenuEnums.EditMenu;
import net.ajaest.lib.utils.SystemEnums.ISO639ー1;

//TODO: JavaDoc
public class SwingMessages {

	private final static Integer CONTEXT_MENU_EDIT = 1;

	public static SwingMessages getLocalizedMessages(ISO639ー1 lang) {

		return new SwingMessages(lang);
	}

	public static SwingMessages getLocalizedMessages(Locale l) {

		return new SwingMessages(ISO639ー1.parse(l.getLanguage().toUpperCase()));
	}

	private final Map<Integer, EnumMap<?, String>> messagesMaps;

	protected ISO639ー1 lang;

	private SwingMessages(ISO639ー1 lang) {
		this.messagesMaps = new HashMap<Integer, EnumMap<?, String>>();
		this.lang = lang;
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public Map<SwingMessagesEnums.ContextMenuEnums.EditMenu, String> getContextMenuEditMessages() {


		EnumMap contextMenuMessages = this.messagesMaps.get(CONTEXT_MENU_EDIT);

		if (contextMenuMessages == null) {
			this.initMessages(CONTEXT_MENU_EDIT);
			contextMenuMessages = this.messagesMaps.get(CONTEXT_MENU_EDIT);
		}

		return contextMenuMessages;
	}

	public ISO639ー1 getCurrentLanguage() {
		return this.lang;
	}

	// TODO: from local xml
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void initMessages(Integer contextMenuEdit) {

		EnumMap messages;

		switch (contextMenuEdit) {
		case 1 : // CONTEXT_MENU_EDIT
		{
			messages = new EnumMap<EditMenu, String>(EditMenu.class);
			switch (this.lang) {
			case ES :
				messages.put(SwingMessagesEnums.ContextMenuEnums.EditMenu.COPY, "Copiar");
				messages.put(SwingMessagesEnums.ContextMenuEnums.EditMenu.CUT, "Cortar");
				messages.put(SwingMessagesEnums.ContextMenuEnums.EditMenu.PASTE, "Pegar");
				messages.put(SwingMessagesEnums.ContextMenuEnums.EditMenu.SELECT_ALL, "Seleccionar Todo");
				break;
			default : // EN
				messages.put(SwingMessagesEnums.ContextMenuEnums.EditMenu.COPY, "Copy");
				messages.put(SwingMessagesEnums.ContextMenuEnums.EditMenu.CUT, "Cut");
				messages.put(SwingMessagesEnums.ContextMenuEnums.EditMenu.PASTE, "Paste");
				messages.put(SwingMessagesEnums.ContextMenuEnums.EditMenu.SELECT_ALL, "Select All");
				break;
			}
			this.messagesMaps.put(CONTEXT_MENU_EDIT, messages);
			break;
		}
		default :
			throw new IllegalArgumentException("Not supported");
		}
	}

}
