//    Copyright (C) 2011  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.lib.math;

import java.util.BitSet;

//TODO: javadoc
public class Random {

    /**
     * Returns a random integer number between floor(inclusive) and ceiling(exclusive)
     */
    public static int generateNumber(int floor, int ceiling) {

        if (floor == ceiling)
            return floor;

        int generated = ceiling;
        while (generated == ceiling) {
            generated = (int) Math.floor(floor + Math.random() * (ceiling - floor));
        }

        return generated;
    }

    /**
     * Generates a randomly generated {@link BitSet} with the specified size and
     * the specified percentage of bits set to 1. If {@code setPercentaje>0.0},
     * at least one bit is set to 1.
     */
    public static BitSet generateRandomBitDistribution(int bitSetSize, float setPercentaje) {

        if ((setPercentaje < 0.0) || (setPercentaje > 1.0)) {
            throw new IllegalArgumentException(
            "Visible percentaje must be between 0(inclusive) and 1(inclusive)");
        }

        BitSet bitSet = new BitSet(bitSetSize);

        if (setPercentaje != 0.0) {
            int randomOffset = generateNumber(0, bitSetSize);
            do {
                while (bitSet.get(randomOffset))
                    randomOffset = generateNumber(0, bitSetSize);
                bitSet.set(randomOffset);
            } while ((((float) bitSet.cardinality()) / ((float) bitSetSize)) < setPercentaje);
        }

        return bitSet;
    }

    public static <T> T[] swapPositionsRandomly(T[] array){

        T temp;
        int randomOffSet;

        for(int i=0; i<array.length; i++){
            randomOffSet = generateNumber(0, array.length);
            temp = array[i];
            array[i] = array[randomOffSet];
            array[randomOffSet] = temp;
        }

        return array;
    }

    public static void main(String... args) {
        Integer generated;
        for (int i = 0; i < 1000; i++) {
            generated = Random.generateNumber(20, 40);
            if ((generated == 40))
                System.err.println(generated);
        }
    }
}
