//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.lib.swing.panel;

import java.awt.Graphics;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JPanel;

import net.ajaest.lib.data.Pair;
import net.ajaest.lib.swing.util.JLineDraw;

/**
 * 
 * This panels allows the user to draw straight lines onto it's canvas surface
 * 
 * There exists a known error in some VM, see <a href=
 * "http://stackoverflow.com/questions/4394206/changed-mouse-event-
 * handling-from-java-1-5-to-1-6">this</a>.
 */
public class JDrawPanel extends JPanel {
    private class LineDrawingKeyAdapter extends KeyAdapter {

        private final JDrawPanel dp;

        public LineDrawingKeyAdapter(JDrawPanel dp) {
            this.dp = dp;
        }

        @Override
        public void keyPressed(KeyEvent k) {

            if (KeyEvent.getKeyText(k.getKeyCode()).equals("Escape")) {
                this.dp.reset();

            }
        }
    }

    private class LineDrawingMouseAdapter extends MouseAdapter {

        private List<Integer> bufferX1;// for backtracking
        private List<Integer> bufferY1;
        private List<Integer> bufferX2;
        private List<Integer> bufferY2;
        private List<Pair<Integer, Integer>> bufferStrokes;

        JDrawPanel dp;

        private boolean mouseClicked = false, savedStateValid = false;

        public LineDrawingMouseAdapter(JDrawPanel dp) {
            this.dp = dp;

        }

        @SuppressWarnings("unused")
        public boolean isMouseClicked() {
            return this.mouseClicked;
        }

        @Override
        public void mouseMoved(MouseEvent me) {

            if (this.mouseClicked) {
                this.dp.setTempX2(me.getX());
                this.dp.setTempY2(me.getY());
                this.dp.repaint();
            }
        }

        /**
         * Be careful with <a href=
         * "http://stackoverflow.com/questions/4394206/changed-mouse-event-
         * handling-from-java-1-5-to-1-6">this</a>.
         */
        @Override
        public void mousePressed(MouseEvent me) {
            this.savedStateValid = false;
            if (me.getButton() == MouseEvent.BUTTON2) {
                this.dp.reset();
            }

            if (!this.mouseClicked && (me.getButton() == MouseEvent.BUTTON1)) {

                this.dp.setTempX1(me.getX());
                this.dp.setTempY1(me.getY());
                this.dp.getX1().add(me.getX());
                this.dp.getY1().add(me.getY());
                this.dp.addMouseMotionListener(this);
                this.mouseClicked = !this.mouseClicked;
                this.dp.setStrokePoint();

            } else if (this.mouseClicked && (me.getButton() == MouseEvent.BUTTON3)) {

                this.dp.removeMouseMotionListener(this);
                // dp.setTempX1(me.getX());
                // dp.setTempY1(me.getY());
                this.dp.getX2().add(me.getX());
                this.dp.getY2().add(me.getY());
                this.mouseClicked = !this.mouseClicked;
                this.dp.setStrokePoint();

                if (!this.savedStateValid) {

                    // removes incomplete line
                    if (JDrawPanel.this.x1.size() > JDrawPanel.this.x2.size()) {
                        JDrawPanel.this.x1.remove(JDrawPanel.this.x1.size() - 1);
                        JDrawPanel.this.y1.remove(JDrawPanel.this.y1.size() - 1);
                    } else if (JDrawPanel.this.x1.size() < JDrawPanel.this.x2.size()) {
                        // TODO: is this possible?
                        JDrawPanel.this.x2.remove(JDrawPanel.this.x2.size() - 1);
                        JDrawPanel.this.y2.remove(JDrawPanel.this.y2.size() - 1);
                    }

                    this.bufferX1 = new ArrayList<Integer>(this.dp.getX1());
                    this.bufferX2 = new ArrayList<Integer>(this.dp.getX2());
                    this.bufferY1 = new ArrayList<Integer>(this.dp.getY1());
                    this.bufferY2 = new ArrayList<Integer>(this.dp.getY2());
                    this.bufferStrokes = new ArrayList<Pair<Integer, Integer>>(this.dp.strokes);
                    this.savedStateValid = true;

                }

            } else if (this.mouseClicked && (me.getButton() == MouseEvent.BUTTON1)) {

                this.dp.getX2().add(me.getX());
                this.dp.getY2().add(me.getY());
                this.dp.repaint();
                this.dp.setTempX1(me.getX());
                this.dp.setTempY1(me.getY());
                this.dp.getX1().add(me.getX());
                this.dp.getY1().add(me.getY());
            }

        }

        @Override
        public void mouseWheelMoved(MouseWheelEvent w) {

            JDrawPanel.this.m.setMouseClicked(false);
            JDrawPanel.this.removeMouseMotionListener(JDrawPanel.this.m);

            int t = w.getWheelRotation();

            if (t > 0) {
                // WHEEL to user remove lines
                this.dp.removeLastLine();
                if (JDrawPanel.this.x1.size() > 0) {
                    // chapuza! The last line is repainted using tempX and tempY
                    this.dp.setTempX1(this.dp.getX1().get(this.dp.getX1().size() - 1));
                    this.dp.setTempY1(this.dp.getY1().get(this.dp.getY1().size() - 1));
                    this.dp.setTempX2(this.dp.getX2().get(this.dp.getX2().size() - 1));
                    this.dp.setTempY2(this.dp.getY2().get(this.dp.getY2().size() - 1));
                }

            }

            else if ((t < 0) && (this.bufferX1.size() > this.dp.getX1().size())) {

                // WHEEL to the other side of the user restores lines
                this.dp.addLine(this.bufferX1.get(this.dp.x1.size()), this.bufferY1.get(this.dp.x1
                        .size()), this.bufferX2.get(this.dp.x1.size()), this.bufferY2
                        .get(this.dp.x1.size()));
                this.dp.setTempX1(this.dp.getX1().get(this.dp.getX1().size() - 1));
                this.dp.setTempY1(this.dp.getY1().get(this.dp.getY1().size() - 1));
                this.dp.setTempX2(this.dp.getX2().get(this.dp.getX2().size() - 1));
                this.dp.setTempY2(this.dp.getY2().get(this.dp.getY2().size() - 1));

                // stroke list must be restored manually
                if (this.dp.strokes.size() > 0){

                    Pair<Integer, Integer> currentStroke = this.dp.strokes.get(this.dp.strokes.size() - 1);
                    Pair<Integer, Integer> restoreStroke = this.bufferStrokes.get(this.dp.strokes.size() - 1);

                    if(	((this.dp.getX1().size()-1) >= restoreStroke.getFirst()) &&
                            ((this.dp.getX1().size()-1) <= restoreStroke.getSecond())){

                        currentStroke.setSecond(currentStroke.getSecond()+1);
                    } else {
                        currentStroke = new Pair<Integer, Integer>(this.dp.getX1().size() - 1, this.dp.getX1().size() - 1);
                        JDrawPanel.this.strokes.add(currentStroke);
                    }
                } else {
                    JDrawPanel.this.strokes.add(new Pair<Integer, Integer>(0, 0));
                }
            }

            this.dp.repaint();
        }

        public void setMouseClicked(boolean mouseClicked) {
            this.mouseClicked = mouseClicked;
        }
    }
    // TODO: this class has a lot of redundant code. I think that in the future
    // it must be burn to ashes and be rebuild.
    /**
     * 
     */
    private static final long serialVersionUID = 7828389152957873184L;

    private final LineDrawingMouseAdapter m;
    private final LineDrawingKeyAdapter k;
    private List<Integer> x1 = new ArrayList<Integer>();

    private List<Integer> y1 = new ArrayList<Integer>();

    private List<Integer> x2 = new ArrayList<Integer>();

    private List<Integer> y2 = new ArrayList<Integer>();

    private List<Pair<Integer, Integer>> strokes = new ArrayList<Pair<Integer, Integer>>();

    private int tempX1, tempY1, tempX2, tempY2;

    public JDrawPanel() {

        this.m = new LineDrawingMouseAdapter(this);
        this.k = new LineDrawingKeyAdapter(this);
        this.addKeyListener(this.k);
        this.addMouseListener(this.m);
        this.addMouseWheelListener(this.m);
    }

    public void addLine(Integer x1, Integer y1, Integer x2, Integer y2) {

        this.x1.add(x1);
        this.x2.add(x2);
        this.y1.add(y1);
        this.y2.add(y2);
        this.tempX1 = -1;
        this.tempX2 = -1;
        this.tempY1 = -1;
        this.tempY2 = -1;
        this.repaint();
    }

    public JLineDraw getJLineDraw(){

        JLineDraw jld;
        if ((this.x1.size() > 0) && (this.y1.size() > 0) && (this.x2.size() > 0) && (this.y2.size() > 0))
            jld = new JLineDraw(this.x1, this.y1, this.x2, this.y2, this.strokes);
        else
            jld = null;
        return jld;
    }

    public List<Pair<Integer, Integer>> getStrokes() {
        return this.strokes;
    }

    private List<Integer> getX1() {
        return this.x1;
    }

    private List<Integer> getX2() {
        return this.x2;
    }

    private List<Integer> getY1() {
        return this.y1;
    }

    private List<Integer> getY2() {
        return this.y2;
    }

    public boolean isMouseClicked() {
        return this.m.mouseClicked;
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        for (int i = 0; i < this.x1.size() - 1; i++)
            g.drawLine(this.x1.get(i), this.y1.get(i), this.x2.get(i), this.y2.get(i));
        if ((this.tempX1 != -1) && (this.tempY1 != -1) && (this.tempX2 != -1) && (this.tempY2 != -1))
            g.drawLine(this.tempX1, this.tempY1, this.tempX2, this.tempY2);
    }

    public void removeLastLine() {
        this.removeMouseMotionListener(this.m);
        this.m.setMouseClicked(false);
        this.tempX1 = -1;
        this.tempX2 = -1;
        this.tempY1 = -1;
        this.tempY2 = -1;

        // deletes the last incomplete vector entry
        if (this.x1.size() > this.x2.size()) {
            this.x1.remove(this.x1.size() - 1);
            this.y1.remove(this.y1.size() - 1);
        } else if (this.x1.size() < this.x2.size()) {
            this.x2.remove(this.x2.size() - 1);
            this.y2.remove(this.y2.size() - 1);
        }

        // now all lists must have same size
        if (this.x1.size() > 0) {
            this.x1.remove(this.x1.size() - 1);
            this.x2.remove(this.x2.size() - 1);
            this.y1.remove(this.y1.size() - 1);
            this.y2.remove(this.y2.size() - 1);

            if (this.x1.size() > 0) {
                // chapuza! The last line is repainted using tempX and tempY
                this.setTempX1(this.getX1().get(this.getX1().size() - 1));
                this.setTempY1(this.getY1().get(this.getY1().size() - 1));
                this.setTempX2(this.getX2().get(this.getX2().size() - 1));
                this.setTempY2(this.getY2().get(this.getY2().size() - 1));
            }

            // rebuild stroke list
            if (this.strokes.size() != 0) {
                Pair<Integer, Integer> p = this.strokes.get(this.strokes.size() - 1);
                if (p.getFirst() > (this.x1.size() - 1))
                    this.strokes.remove(p);
                else {
                    // A new pair must be created in order to keep restore
                    // buffer
                    // integrity
                    this.strokes.set(this.strokes.size() - 1, new Pair<Integer, Integer>(p.getFirst(), this.x1.size() - 1));
                }
            }
        }

        this.repaint();
    }

    public void removeLastStroke() {

        if (this.strokes.size() > 0) {
            Pair<Integer, Integer> lastStroke = this.strokes.get(this.strokes.size() - 1);
            Integer removeLines = lastStroke.getSecond() - lastStroke.getFirst() + 1;

            for (int i = 0; i < removeLines; i++)
                this.removeLastLine();

            this.strokes.remove(lastStroke);
        }
    }

    public void reset() {

        this.unclickMouse();

        this.x1 = new ArrayList<Integer>();
        this.x2 = new ArrayList<Integer>();
        this.y1 = new ArrayList<Integer>();
        this.y2 = new ArrayList<Integer>();
        this.strokes = new ArrayList<Pair<Integer, Integer>>();
        this.tempX1 = -1;
        this.tempX2 = -1;
        this.tempY1 = -1;
        this.tempY2 = -1;
        this.removeMouseMotionListener(this.m);


        this.repaint();
    }

    private void setStrokePoint() {

        // first element of the pair is the beginning line of the stroke
        // and second element is the end of the stroke
        Pair<Integer, Integer> tempPair;

        if (this.strokes.size() == 0)
            this.strokes.add(new Pair<Integer, Integer>(this.x1.size() - 1, null));
        else {

            tempPair = this.strokes.get(this.strokes.size() - 1);

            if (tempPair.getSecond() == null) {
                tempPair.setSecond(this.x1.size() - 1);
            }
            else
                this.strokes.add(new Pair<Integer, Integer>(this.x1.size() - 1, null));
        }

    }

    private void setTempX1(int tempX1) {
        this.tempX1 = tempX1;
    }

    private void setTempX2(int tempX2) {
        this.tempX2 = tempX2;
    }

    private void setTempY1(int tempY1) {
        this.tempY1 = tempY1;
    }

    private void setTempY2(int tempY2) {
        this.tempY2 = tempY2;
    }

    public void unclickMouse() {

        // TODO: WARNING! check if exists problem with strokes when unclick is
        // used or when last line is removed with scroll when it's being formed
        this.m.setMouseClicked(false);
        this.removeMouseMotionListener(this.m);

        // removes incomplete line
        if (this.x1.size() > this.x2.size()) {
            this.x1.remove(this.x1.size() - 1);
            this.y1.remove(this.y1.size() - 1);
        } else if (this.x1.size() < this.x2.size()) {
            // TODO: is this possible?
            this.x2.remove(this.x2.size() - 1);
            this.y2.remove(this.y2.size() - 1);
        }

        if (this.x1.size() > 0) {
            // chapuza! The last line is repainted using tempX and tempY
            this.setTempX1(this.getX1().get(this.getX1().size() - 1));
            this.setTempY1(this.getY1().get(this.getY1().size() - 1));
            this.setTempX2(this.getX2().get(this.getX2().size() - 1));
            this.setTempY2(this.getY2().get(this.getY2().size() - 1));
        }

        this.setStrokePoint();

        this.m.bufferX1 = new ArrayList<Integer>(this.getX1());
        this.m.bufferX2 = new ArrayList<Integer>(this.getX2());
        this.m.bufferY1 = new ArrayList<Integer>(this.getY1());
        this.m.bufferY2 = new ArrayList<Integer>(this.getY2());
        this.m.bufferStrokes = new ArrayList<Pair<Integer, Integer>>(this.strokes);
        this.m.savedStateValid = true;

        this.repaint();
    }
}
