//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.lib.util.tools;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import java.util.TreeSet;

import net.ajaest.lib.util.comparators.FileNameComparator;

//TODO: JavaDoc
public class TextDocumentBuilder {

	private static String version = "$version";
	private static String project = "$project";
	private static String resourceUri = "$resourceUri";

	private static String enconding = "UTF-8";
	private static String outputFile = "result";

	private static Queue<File> concQueue = null;
	private static List<String> concFiles = null;

	private static StringBuilder result = null;


	/**
	 * @param args
	 */
	public static void main(String[] args) {

		concFiles = new ArrayList<String>();

		for (String s : args) {
			if (s.startsWith("-_version="))
				version = s.replace("-_version=", "");
			else if (s.startsWith("-_project="))
				project = s.replace("-_project=", "");
			else if (s.startsWith("-_resourceUri="))
				resourceUri = s.replace("-_resourceUri=", "");
			else if (s.startsWith("-enc="))
				enconding = s.replace("-enc=", "");
			else if (s.startsWith("-out="))
				outputFile = s.replace("-out=", "");
			else
				concFiles.add(s);
		}

		if (concFiles.size() == 0) {
			System.out.println("No work to do.");
			System.exit(0);
		}

		try {
			buildProcessingQueue();

			result = new StringBuilder();

			File current;
			while ((current = concQueue.poll()) != null)
				result.append(getFileText(current));

		} catch (Exception e) {
			System.err.println("**Error when trying to retrieve text: " + e.getLocalizedMessage());
			System.exit(-1);
		}

		String strResult = result.toString();

		strResult = strResult.replace("($version)", version);
		strResult = strResult.replace("($project)", project);
		strResult = strResult.replace("($resourceUri)", resourceUri);

		try {
			BufferedWriter bf = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(new File(outputFile)), enconding));
			bf.write(strResult);
			bf.flush();
			bf.close();
		} catch (Exception e) {
			System.err.println("**Error when trying to write output file: " + e.getLocalizedMessage());
			System.exit(-1);
		}

		System.out.println("Output file\"" + outputFile + "\" was concatenated successfully");

		System.exit(0);
	}

	private static void buildProcessingQueue() {

		concQueue = new LinkedList<File>();
		for (String path : concFiles)
			concQueue.addAll(makeFileQueue(path));
	}

	private static Queue<File> makeFileQueue(String path) {

		Queue<File> fileQueue = new LinkedList<File>();

		File f = new File(path);
		
		
		if (f.isDirectory()) {

			TreeSet<File> childs = new TreeSet<File>(new FileNameComparator());
			Collections.addAll(childs, f.listFiles());
			for (File child : childs)
				if (child.isFile())
					fileQueue.add(child);
		} else
			fileQueue.add(f);

		return fileQueue;

	}

	private static StringBuilder getFileText(File path) throws IOException {

		StringBuilder text = new StringBuilder("");

		BufferedReader bis = new BufferedReader(new FileReader(path));

		String line;
		while ((line = bis.readLine()) != null) {
			text.append(line + "\r\n");
		}

		bis.close();

		return text;

	}

}
