//    Copyright (C) 2011  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.lib.xml;

import java.util.ArrayList;
import java.util.List;

import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

//TODO: javadoc
public class ValidationReport implements ErrorHandler {

	private final List<String> results;
	private final List<String> warnings;
	private final List<String> errors;
	private final List<String> fatalerrors;

	public ValidationReport() {
		this.results = new ArrayList<String>();
		this.warnings = new ArrayList<String>();
		this.errors = new ArrayList<String>();
		this.fatalerrors = new ArrayList<String>();
	}

	@Override
	public void warning(SAXParseException e) throws SAXException {

		String message = this.buildMessage("warning", e);

		this.results.add(message);
		this.warnings.add(message);
	}

	@Override
	public void error(SAXParseException e) throws SAXException {

		String message = this.buildMessage("error", e);

		this.results.add(message);
		this.errors.add(message);
	}

	@Override
	public void fatalError(SAXParseException e) throws SAXException {

		String message = this.buildMessage("fatalerror", e);

		this.results.add(message);
		this.fatalerrors.add(message);
	}

	private String buildMessage(String type,SAXParseException e ){
		String message = e.getColumnNumber() + ":" + e.getLineNumber() + ", " + type + ": " +e.getMessage();

		return message;
	}

	public List<String> getResults() {
		return this.results;
	}

	public List<String> getWarnings() {
		return this.warnings;
	}

	public List<String> getErrors() {
		return this.errors;
	}

	public List<String> getFatalErrors() {
		return this.fatalerrors;
	}

	public Boolean isValid(){
		return (this.getErrors().size()==0) && (this.getFatalErrors().size()==0);
	}

}
