//    Copyright (C) 2011  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.lib.xml;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import nu.xom.Builder;
import nu.xom.DocType;
import nu.xom.Document;
import nu.xom.ParsingException;
import nu.xom.Serializer;
import nu.xom.ValidityException;

import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * Utility class for XML
 * 
 * @author Luis A. Arce
 * 
 */
public class XMLUtils {

	/**
	 * Validates a XML document against
	 * 
	 * @param doc
	 * @return
	 * @throws IOException
	 * @throws SAXException
	 * @throws ParserConfigurationException
	 * @throws ParsingException
	 * @throws ValidityException
	 */
	public static ValidationReport validAgainstExternalDTD(nu.xom.Document doc, String dtdPath)
	throws ParsingException {



		Document tempD = new Document(doc);

		DocType wplDT = new DocType(tempD.getRootElement().getLocalName(), dtdPath);
		tempD.setDocType(wplDT);

		ValidationReport isWPL = new ValidationReport();
		File tempFile;
		try {
			tempFile = File.createTempFile("valwpl", "xml");

			tempFile.deleteOnExit();

			BufferedOutputStream tempStream = new BufferedOutputStream(
					new FileOutputStream(tempFile));
			Serializer slr = new Serializer(tempStream, "UTF-8");
			slr.write(tempD);
			slr.flush();
			tempStream.close();

			SAXParserFactory factory = SAXParserFactory.newInstance();
			factory.setValidating(true);
			factory.setNamespaceAware(true);

			SAXParser parser;
			XMLReader reader;

			parser = factory.newSAXParser();
			reader = parser.getXMLReader();


			reader.setErrorHandler(isWPL);

			Builder builder = new Builder(reader);

			builder.build(tempFile);
			tempFile.delete();
		} catch (ValidityException e) {
		} catch (Exception e1) {
			ParsingException pe = new ParsingException(e1.getLocalizedMessage(), e1);
			throw pe;
		}

		return isWPL;
	}

	public static void main(String... args) throws ValidityException, ParsingException, IOException {
		Document d = new Builder().build(new File("/home/ajaest11/.local/share/rhythmbox/rhythmdb.xml"));

		ValidationReport vr = validAgainstExternalDTD(d, "/home/ajaest11/Codigo/PuraiConverter/trunk/net/ajaest/purai/validation/RhytmBox.dtd");

		System.out.println(vr.isValid());
		for(String s: vr.getResults())
			System.out.println(s);
	}
}
